import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class MainView extends JFrame implements ActionListener{

  private static final long serialVersionUID = 1L;
  private JTextArea codetext = new JTextArea();
  private JScrollPane codepane = new JScrollPane(codetext);
  private JPanel copypane = new JPanel();
  private JPanel rightpane = new JPanel();
  private JPanel leftpane = new JPanel();
  private JButton openButton = new JButton("Datei ffnen");
  private JPanel[] pinPanels = new JPanel[14];
  private JComboBox[] pinCombos = new JComboBox[13];
  private String[] comboContent = {"unbenutzt", "Eingang", "Ausgang" };
  private JTextArea openedfile = new JTextArea();
  private JButton generateButton = new JButton("Generiere Quelltext");
  private JButton copyButton = new JButton("Kopiere in Zwischenablage");
  final JFileChooser fc = new JFileChooser();
  
  public MainView(){
    this.setDefaultCloseOperation(EXIT_ON_CLOSE);
    this.setTitle("UML2Arduino");
    setComponents();
    this.pack();
  }
  
  //sets the cmoponents to the JFrame
  private void setComponents(){
    fc.addChoosableFileFilter(new XmlFilter());
    this.getContentPane().setLayout(new FlowLayout());
    codepane.setPreferredSize(new Dimension(600,600));
    this.getContentPane().add(leftpane);
    this.getContentPane().add(rightpane);
    leftpane.setLayout(new BoxLayout(leftpane, BoxLayout.Y_AXIS));
    rightpane.setLayout(new BoxLayout(rightpane, BoxLayout.Y_AXIS));
    rightpane.add(codepane);
    rightpane.add(copypane);
    FlowLayout fl = new FlowLayout();
    fl.setAlignment(FlowLayout.RIGHT);
    copypane.setLayout(fl);
    copypane.add(copyButton);
    openedfile.append("Keine Datei geladen");
    JPanel openPanel = new JPanel();
    openPanel.setLayout(new GridLayout(0,1));
    openPanel.add(openButton);
    openPanel.add(openedfile);
    leftpane.add(openPanel);
    for (int i=0; i<13; i++){
      pinPanels[i]=new JPanel();
      pinPanels[i].setLayout(new FlowLayout());
      pinPanels[i].add(new JLabel("Pin "+(i+1)));
      pinCombos[i] = new JComboBox(comboContent);
      pinPanels[i].add(pinCombos[i]);
      
      leftpane.add(pinPanels[i]);
    }
    generateButton.setEnabled(false);
    JPanel generatePanel = new JPanel();
    generatePanel.setLayout(new FlowLayout());
    generatePanel.add(generateButton);
    leftpane.add(generatePanel);
    codetext.setEditable(false);
    openButton.addActionListener(this);
    generateButton.addActionListener(this);
    copyButton.addActionListener(this);
  }

  @Override
  public void actionPerformed(ActionEvent arg0) {
    if (arg0.getSource()==openButton){
      int returnVal = fc.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        Parser p = new Parser(fc.getSelectedFile().getPath());
        Exception e = p.parse();
        if (e!=null){
          if (e instanceof NotUmlException) {
            JOptionPane.showMessageDialog(this, e.toString());
            generateButton.setEnabled(true);
            openedfile.setText(fc.getSelectedFile().getName()+" wurde geladen.");
          }
          if (e instanceof IOException)
            JOptionPane.showMessageDialog(this, "Datei konnte nicht geladen werden.");
        }
        else {
          generateButton.setEnabled(true);
          openedfile.setText(fc.getSelectedFile().getName()+" wurde geladen.");
        }
      }
      this.pack();
    }
    
    if (arg0.getSource()==generateButton){
      codetext.setText("");
      String[] pinmodes = new String[13];
      for (int i=0;i<13;i++){
        pinmodes[i]=(String)pinCombos[i].getSelectedItem();
      }
      codetext.setText(StateDiagram.getIntance().generateCode(pinmodes));
      codetext.setCaretPosition(0);
      if (StateDiagram.getIntance().getError())
        JOptionPane.showMessageDialog(this, "Hinweis: Nicht alle Transitionen konnten einwandfrei erkannt werden!");
      
    }
    
    if (arg0.getSource()==copyButton){
      Toolkit.getDefaultToolkit().getSystemClipboard().setContents(
          new StringSelection(codetext.getText()), null
      );
    }
  }
  
}